library comapi32;

uses Windows;

var
  hCom : THANDLE;         //COM handle
  dcb  : TDCB;            //COM device control block
  ComTo: TCOMMTIMEOUTS;   //COM timeouts

//Access check for COM resource

function AccessCOM(P:PChar):boolean; stdcall;
var hTemp: THANDLE;
begin
 hTemp:= CreateFile(P,
                    GENERIC_READ or GENERIC_WRITE,
                    0,
                    nil,
                    OPEN_EXISTING,
                    0,
                    0);
 if not (hTemp=INVALID_HANDLE_VALUE)
  then begin CloseHandle(hTemp); Result:=true;  end
  else Result:=false;
end;

//Open COM for non overlapped operations

function OpenCOM(P:PChar):boolean; stdcall;
begin
 hCom:= CreateFile(P,
                   GENERIC_READ or GENERIC_WRITE,
                   0,
                   nil,
                   OPEN_EXISTING,
                   0,
                   0);
 if (hCom=INVALID_HANDLE_VALUE)
  then begin Result:=false; exit; end;
 if not GetCommState(hCom,dcb)
  then begin Result:=false; exit; end;
 if not GetCommTimeouts(hCom,ComTo)
  then begin Result:=false; exit; end;
 Result:=true;
end;

//Close COM

function CloseCOM:boolean; stdcall;
begin
 Result:=CloseHandle(hCom);
end;

//Set COM baud rate, byte size, parity, stop bits

function SetCOM(baud:DWORD;bsize,parity,
                sbits:BYTE):boolean; stdcall;
begin
 dcb.BaudRate:=baud;
 dcb.Flags:=$00000001;
 dcb.ByteSize:=bsize;
 dcb.Parity:=parity;
 dcb.StopBits:=sbits;
 Result:=SetCommState(hCom,dcb);
end;

//Set COM baud rate, byte size, parity, stop bits
// and set/clear DTR/RTS

function SetModCOM(baud:DWORD;bsize,parity,
                sbits:BYTE;DTR,RTS:boolean):boolean; stdcall;
begin
 dcb.BaudRate:=baud;
 dcb.Flags:=$00000001;
 if DTR then dcb.Flags:=dcb.Flags or $00000010;
 if RTS then dcb.Flags:=dcb.Flags or $00001000;
 dcb.ByteSize:=bsize;
 dcb.Parity:=parity;
 dcb.StopBits:=sbits;
 Result:=SetCommState(hCom,dcb);
end;

//Set COM timeouts

function SetCOMTo(RdIvl,RdTotM,RdTotC,
          WrTotM,WrTotC:DWORD):boolean; stdcall;
begin
 ComTo.ReadIntervalTimeout:=RdIvl;
 ComTo.ReadTotalTimeoutMultiplier:=RdTotM;
 ComTo.ReadTotalTimeoutConstant:=RdTotC;
 ComTo.WriteTotalTimeoutMultiplier:=WrTotM;
 ComTo.WriteTotalTimeoutConstant:=WrTotC;
 Result:=SetCommTimeouts(hCom,ComTo);
end;

//Set COM buffers

function SetCOMBuff(RdBuff,WrBuff:DWORD):boolean; stdcall;
begin
 Result:=SetupComm(hCom,RdBuff,WrBuff);
end;

//Escape COM Functions

function EscFuncCOM(F:DWORD):boolean; stdcall;
begin
 Result:=EscapeCommFunction(hCom,F)
end;

//Get modem status

function GetModem(var lpD:DWORD):boolean; stdcall;
begin
 Result:=GetCommModemStatus(hCom,lpD);
end;

//Purge COM: terminates TX and RX and clears buffers

function PurgeCOM:boolean; stdcall;
begin
 Result:=PurgeComm(hCom,PURGE_TXABORT or
                        PURGE_RXABORT or
                        PURGE_TXCLEAR or
                        PURGE_RXCLEAR);
end;

//Flush COM: TX buffer TX and clear then return

function FlushCOM:boolean; stdcall;
begin
 Result:=FlushFileBuffers(hCom);
end;

//Tx byte ahead

function TxByteCOM(data:byte):boolean; stdcall;
begin
 Result:=TransmitCommChar(hCom,Chr(data));
end;

//Tx N bytes from Buffer in non overlapped mode

function TxDataCOM(const Buffer; N:DWORD;
                   var lpNDone:DWORD):boolean; stdcall;
begin
  Result:=WriteFile(hCom,Buffer,N,lpNDone,nil);
end;

//Rx N bytes to Buffer in non overlapped mode

function RxDataCOM(var Buffer; N:DWORD;
                   var lpNDone:DWORD):boolean; stdcall;
begin
  Result:=ReadFile(hCom,Buffer,N,lpNDone,nil);
end;

//Clear COM error flag and report

function ClrErrCOM(var lpE:DWORD):boolean; stdcall;
begin
  Result:=ClearCommError(hCom,lpE,nil);
end;

//Get COM events mask

function GetMaskCOM(var lpEvtMask:DWORD):boolean; stdcall;
begin
  Result:=GetCommMask(hCom,lpEvtMask);
end;

//Set COM events mask

function SetMaskCOM(EvtMask:DWORD):boolean; stdcall;
begin
  Result:=SetCommMask(hCom,EvtMask);
end;

//Wait COM event

function WaitEventCOM(var lpEvtMask:DWORD):boolean; stdcall;
begin
  Result:=WaitCommEvent(hCom,lpEvtMask,nil);
end;

exports
 AccessCOM,
 OpenCOM,
 CloseCOM,
 SetCOM,
 SetModCOM,
 SetCOMTo,
 SetCOMBuff,
 EscFuncCOM,
 GetModem,
 PurgeCOM,
 FlushCOM,
 TxByteCOM,
 TxDataCOM,
 RxDataCOM,
 ClrErrCOM,
 GetMaskCOM,
 SetMaskCOM,
 WaitEventCOM;

begin
end.
